# Generated by Django 2.1.5 on 2019-02-07 12:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('netkeiba', '0002_add_models'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='race',
            name='grade',
        ),
    ]
